{% extends "registration/base.html" %}
{% load common i18n %}

{% block content %}
	<h1>{% block page_title %}{% trans "Password reset" %}{% endblock %}</h1>

	<p>{% trans "Forgotten your password? Enter your e-mail address below, and we'll e-mail instructions for setting a new one." %}</p>

	<form class="fields" action="" method="post">
		{% csrf_token %}

		{% for f in form %}
			{% field f %}
		{% endfor %}

		<div class="btn-group">
			<button type="submit">
				<i class="fa fa-play-circle"></i>
				{% trans "Reset my password" %}
			</button>
		</div>
	</form>
{% endblock %}
