# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-27 21:39
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0027_matchvideo_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='matchvideo',
            name='match',
        ),
        migrations.DeleteModel(
            name='MatchVideo',
        ),
    ]
