<!DOCTYPE html>
<html>
<head lang="en">
	<meta charset="UTF-8">
	<title></title>
</head>
<body>

<p>{{ matches.0.count }} : {{ matches.1.count }}</p>
<p>{{ teams.0|length }} : {{ teams.1|length }}</p>
<p>{{ clubs.0.count }} : {{ clubs.1.count }}</p>
<p>{{ points.0 }} : {{ points.1 }}</p>
<p>{{ caps.0.caps|default_if_none:"-" }} : {{ caps.1.caps|default_if_none:"-" }}</p>

</body>
</html>