{% extends "tournamentcontrol/competition/admin/_club_teams.html" %}
{% load i18n %}

{% block breadcrumbs %}
	{{ block.super }}
	<li><a href="{% url 'rego:club:list' %}">{% trans "Clubs" %}</a></li>
	<li><a href="{% url 'rego:club:edit' club.pk %}">{{ club.title }}</a></li>
	<li><span>{% trans "Teams" %}</span></li>
{% endblock %}

{% block table_actions %}
	<a href="{% url 'rego:edit-team-members' club.pk object.pk %}">{% trans "Members" %}</a>
{% endblock %}

{% block season_actions %}
	<div class="actions">
		<a class="button report" href="{% url 'rego:officials' club.pk group.grouper.pk %}">{% trans "Officials" %}</a>
		<a class="pdf button" href="{% url 'rego:registration-form' club.pk group.grouper.pk "pdf" %}">{% trans "Form" %}</a>
	</div>
{% endblock %}
